package net.wanji.common.utils.webservice;

import com.alibaba.fastjson.JSON;

import java.util.HashMap;
import java.util.Map;

/**
 * @ClassName SoapUtils
 * @Description
 * @Author mapabc7
 * @Date 2020/6/1 18:20
 * @Version 1.0
 */
public class SoapUtils {
    /**
     * 生成soapxml
     *
     * @param targetNamespace
     * @param functionName
     * @param bodyMap
     * @return
     */
    public static String buildSoapXml(String targetNamespace, String functionName, Map<String, Object> bodyMap) {
        StringBuilder stringBuilder = new StringBuilder("<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"" +
                " xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n")
                .append("\t<soap:Body>\n")
                .append("\t\t<" + functionName + " xmlns=\"" + targetNamespace + "\">\n")
                .append(paramTransformSoapXml(bodyMap))
                .append("\t\t</" + functionName + ">\n")
                .append("\t</soap:Body>\n")
                .append("</soap:Envelope>");
        return String.valueOf(stringBuilder);
    }

    /**
     * 将参数转化为soapXml中的一部分
     *
     * @param paramMap
     * @return
     */
    private static String paramTransformSoapXml(Map<String, Object> paramMap) {
        if (paramMap.isEmpty()) {
            return "";
        }
        StringBuilder stringBuilder = new StringBuilder();
        for (Map.Entry<String, Object> entry : paramMap.entrySet()) {
            stringBuilder.append(paramTransformSoapXml(entry));
        }
        return String.valueOf(stringBuilder);
    }

    private static String paramTransformSoapXml(Map.Entry<String, Object> entry) {
        StringBuilder stringBuilder = new StringBuilder();
        String entryKey = entry.getKey();
        Object entryValue = entry.getValue();
        if (entryValue instanceof Map) {
            stringBuilder.append("\t\t\t").append("<" + entryKey + ">\n");
            stringBuilder.append(paramTransformSoapXml(JSON.parseObject(JSON.toJSONString(entryValue))));
            stringBuilder.append("\t\t\t").append("</" + entryKey + ">\n");
        } else {
            stringBuilder.append("\t\t\t").append("<" + entryKey + ">").append(entryValue).append("</" + entryKey + ">\n");
        }
        return String.valueOf(stringBuilder);
    }

    public static void main(String[] args) {
        String targetNamespace = "http://WebXml.com.cn/";
        String funName = "getSupportCity";
        Map bodyMap = new HashMap() {{
            put("byProvinceName", "河北");
        }};
        String soapXml = SoapUtils.buildSoapXml(targetNamespace, funName, bodyMap);
        System.out.println(soapXml);
    }
}
